// This file was generated by ecm_setup_version(): DO NOT EDIT!

#ifndef LIBKTORRENT_VERSION_H
#define LIBKTORRENT_VERSION_H

#define LIBKTORRENT_VERSION_STRING "21.04.1"
#define LIBKTORRENT_VERSION_MAJOR 21
#define LIBKTORRENT_VERSION_MINOR 4
#define LIBKTORRENT_VERSION_PATCH 1
#define LIBKTORRENT_VERSION ((21<<16)|(4<<8)|(1))

#endif
